#include <sourcemod>
#include <adminmenu>
#include <sdktools>
#include <sdkhooks>
#include <cstrike>

#define PLUGIN_VERSION "1.1"
#define PLUGIN_NAME "Western Dueling"

public Plugin:myinfo =
{
    name = PLUGIN_NAME,
    author = "xbatista",
    description = "Duel like wetern style",
    version = PLUGIN_VERSION,
    url = "www.laikiux.lt"
};

new const String:DUELING_SOUNDS[][] =
{
	"wd/dueling1.mp3"
}

new Handle:DuelStartTimer[MAXPLAYERS + 1] = INVALID_HANDLE;
new Handle:DuelPrepareTimer[MAXPLAYERS + 1] = INVALID_HANDLE;
new Handle:DuelFailTimer[MAXPLAYERS + 1] = INVALID_HANDLE;
new Handle:DuelEndTimer[MAXPLAYERS + 1] = INVALID_HANDLE;

new p_DuelTarget[ MAXPLAYERS + 1 ];
new bool:p_DuelAccepted[ MAXPLAYERS + 1 ];
new bool:p_IsDueling[ MAXPLAYERS + 1 ];

new String:WepBeforeDuel[ MAXPLAYERS + 1 ][ 2 ][ 32 ];

new bool:is_RoundEnded;

new Handle:CvarDuelStartTime, Handle:CvarDuelCheckTimer, Handle:CvarDuelEndTime;

public OnPluginStart()
{
	CreateConVar("wester_dueling", PLUGIN_VERSION, PLUGIN_NAME, FCVAR_PLUGIN|FCVAR_SPONLY|FCVAR_REPLICATED|FCVAR_NOTIFY|FCVAR_REPLICATED);

	CvarDuelStartTime = CreateConVar("wd_starttime", "7.0", "Time when duel begins.")
	CvarDuelEndTime = CreateConVar("wd_endtime", "8.0", "Time when duel ends.")
	CvarDuelCheckTimer = CreateConVar("wd_checktimer", "15.0", "Time until duel invite will fail if player is not in view.")
	
	HookEvent("player_death", Event_PlayerDeath);
	HookEvent("round_start", Event_RoundStartPost);
	HookEvent("round_end", Event_RoundEnd);
	HookEvent("player_team", EventTeamChange, EventHookMode_Pre);

	AddCommandListener( Say_Command, "say");
}

public OnMapStart()
{
	decl String:Snd_Path[256];

	// Dueling Sounds Precache
	for(new iM = 0; iM < sizeof DUELING_SOUNDS; iM++)
	{
		Format( Snd_Path, sizeof(Snd_Path) , "sound/%s", DUELING_SOUNDS[iM])
			
		if( FileExists(Snd_Path) )
		{
			PrecacheSound( DUELING_SOUNDS[iM], true );
			AddFileToDownloadsTable( Snd_Path );
		}
		else
		{
			LogError("Cannot locate file '%s'", Snd_Path)
		}
	}
}

public OnClientPutInServer(client)
{
	DuelStartTimer[ client ] = INVALID_HANDLE;
	DuelEndTimer[ client ] = INVALID_HANDLE;
	DuelPrepareTimer[ client ] = INVALID_HANDLE;
	DuelFailTimer[ client ] = INVALID_HANDLE;

	SDKHook(client, SDKHook_WeaponCanSwitchTo, OnWeaponCanSwitchTo);
	SDKHook(client, SDKHook_WeaponEquip, OnWeaponEquip);
	SDKHook(client, SDKHook_TraceAttack, OnTraceAttack);
	SDKHook(client, SDKHook_OnTakeDamage, OnTakeDamage);
	SDKHook(client, SDKHook_WeaponDrop, OnWeaponDrop);
}
public OnClientDisconnect(client)
{
	p_DuelAccepted[client] = false;
	p_DuelAccepted[p_DuelTarget[client]] = false;

	p_IsDueling[client] = false;
	p_IsDueling[p_DuelTarget[client]] = false;
	
	p_DuelTarget[p_DuelTarget[client]] = 0;
	p_DuelTarget[client] = 0;
	
	if ( DuelStartTimer[ client ] != INVALID_HANDLE )
	{
		CloseHandle( DuelStartTimer[ client ] );
		DuelStartTimer[ client ] = INVALID_HANDLE;
	}
	if ( DuelEndTimer[ client ] != INVALID_HANDLE )
	{
		CloseHandle( DuelEndTimer[ client ] );
		DuelEndTimer[ client ] = INVALID_HANDLE;
	}
	if ( DuelFailTimer[ client ] != INVALID_HANDLE )
	{
		CloseHandle( DuelFailTimer[ client ] );
		DuelFailTimer[ client ] = INVALID_HANDLE;
	}
}

public Action:Event_RoundStartPost(Handle:event, const String:name[], bool:dontBroadcast)
{
	is_RoundEnded = false;
}
public Action:Event_RoundEnd(Handle:event, const String:name[], bool:dontBroadcast)
{
	is_RoundEnded = true;

	for( new i = 1 ; i <= MaxClients ; i++ )
	{
		if( IsValidAlive( i ) )
		{
			p_DuelAccepted[i] = false;
			p_IsDueling[i] = false;
			p_DuelTarget[i] = 0;
			p_DuelTarget[p_DuelTarget[i]] = 0;
			
			SetEntityMoveType(i, MOVETYPE_WALK);
			
			if ( DuelStartTimer[ i ] != INVALID_HANDLE )
			{
				CloseHandle( DuelStartTimer[ i ] );
				DuelStartTimer[ i ] = INVALID_HANDLE;
			}
			if ( DuelEndTimer[ i ] != INVALID_HANDLE )
			{
				CloseHandle( DuelEndTimer[ i ] );
				DuelEndTimer[ i ] = INVALID_HANDLE;
			}
			if ( DuelFailTimer[ i ] != INVALID_HANDLE )
			{
				CloseHandle( DuelFailTimer[ i ] );
				DuelFailTimer[ i ] = INVALID_HANDLE;
			}
		}
	}
}

public Action:EventTeamChange(Handle:event, const String:name[], bool:dontBroadcast)
{
	new client = GetClientOfUserId(GetEventInt(event, "userid"));

	if( IsValidAlive(client) && p_IsDueling[client] )
	{
		return Plugin_Handled;
	}
	
	return Plugin_Continue;
}
public Action:Event_PlayerDeath(Handle:event, const String:name[], bool:dontBroadcast)
{
	new victim = GetClientOfUserId(GetEventInt(event, "userid") );

	if ( !IsValidClient( victim ) || !p_DuelAccepted[victim] ) return Plugin_Continue;
	
	new attacker = p_DuelTarget[victim];

	PrintToChat( victim, "You fail!");
	
	p_DuelAccepted[victim] = false;
	p_IsDueling[victim] = false;
	p_DuelTarget[victim] = 0;
	
	if ( DuelStartTimer[ victim ] != INVALID_HANDLE )
	{
		CloseHandle( DuelStartTimer[ victim ] );
		DuelStartTimer[ victim ] = INVALID_HANDLE;
	}
	if ( DuelEndTimer[ victim ] != INVALID_HANDLE )
	{
		CloseHandle( DuelEndTimer[ victim ] );
		DuelEndTimer[ victim ] = INVALID_HANDLE;
	}
	if ( DuelFailTimer[ victim ] != INVALID_HANDLE )
	{
		CloseHandle( DuelFailTimer[ victim ] );
		DuelFailTimer[ victim ] = INVALID_HANDLE;
	}
	
	if ( IsValidClient( attacker ) )
	{
		PrintToChat( attacker, "You won the duel!");
		
		p_DuelAccepted[attacker] = false;

		p_IsDueling[attacker] = false;
	
		p_DuelTarget[attacker] = 0;
		
		SetEntityMoveType(attacker, MOVETYPE_WALK);

		Strip_Wep_Deagle(attacker);

		if ( !StrEqual(WepBeforeDuel[attacker][CS_SLOT_PRIMARY], "NULL") )
			GivePlayerItem(attacker, WepBeforeDuel[attacker][CS_SLOT_PRIMARY]);
		if ( !StrEqual(WepBeforeDuel[attacker][CS_SLOT_SECONDARY], "NULL") )
			GivePlayerItem(attacker, WepBeforeDuel[attacker][CS_SLOT_SECONDARY]);
		
		if ( DuelStartTimer[ attacker ] != INVALID_HANDLE )
		{
			CloseHandle( DuelStartTimer[ attacker ] );
			DuelStartTimer[ attacker ] = INVALID_HANDLE;
		}
		if ( DuelEndTimer[ attacker ] != INVALID_HANDLE )
		{
			CloseHandle( DuelEndTimer[ attacker ] );
			DuelEndTimer[ attacker ] = INVALID_HANDLE;
		}
		if ( DuelFailTimer[ attacker ] != INVALID_HANDLE )
		{
			CloseHandle( DuelFailTimer[ attacker ] );
			DuelFailTimer[ attacker ] = INVALID_HANDLE;
		}
	}
	
	return Plugin_Continue;
}

public Action:OnWeaponDrop(client, weapon)
{
	if ( !IsValidEntity(weapon) || !IsValidEdict(weapon) || !IsValidAlive(client) ||
	!p_DuelAccepted[client] )
		return Plugin_Continue;

	AcceptEntityInput(weapon, "Kill");
	
	return Plugin_Continue;
}
public Action:OnTraceAttack(victim, &attacker, &inflictor, &Float:damage, &damagetype, &ammotype, hitbox, hitgroup)
{
	if ( !IsValidAlive(victim) || !IsValidAlive(attacker) ) return Plugin_Continue;

	new duel_victim = p_DuelTarget[attacker];
	new duel_attacker = p_DuelTarget[victim];

	if ( p_DuelAccepted[victim] && attacker != duel_attacker || ( p_DuelAccepted[attacker] && victim != duel_victim ) )
		return Plugin_Handled;
		
	if ( p_DuelAccepted[victim] && p_DuelAccepted[duel_attacker] && !p_IsDueling[victim]
	&& !p_IsDueling[duel_attacker] )
		return Plugin_Handled;

	return Plugin_Continue;
}
public Action:OnTakeDamage(victim, &attacker, &inflictor, &Float:damage, &damagetype)
{
	if ( !IsValidAlive(victim) || !IsValidAlive(attacker) ) return Plugin_Continue;
	
	new duel_attacker = p_DuelTarget[victim];
	
	new iTeam = GetClientTeam(victim);
	
	if ( IsValidAlive(duel_attacker) && iTeam == GetClientTeam(duel_attacker) && p_IsDueling[duel_attacker] )
	{
		CS_SwitchTeam( victim, iTeam == CS_TEAM_CT ? CS_TEAM_T : CS_TEAM_CT );
		SDKHooks_TakeDamage(victim, inflictor, duel_attacker, damage, damagetype);
		CS_SwitchTeam( victim, iTeam );
	}

	return Plugin_Continue;
}

public Action:OnWeaponEquip(client, weapon)
{
	if ( !IsValidEntity(weapon) || !IsValidEdict(weapon) || !IsValidAlive(client) ||
	!p_DuelAccepted[client] )
		return Plugin_Continue;

	decl String:sWeapon[32];
	GetEdictClassname(weapon, sWeapon, sizeof(sWeapon));
	
	if ( StrEqual( sWeapon, "weapon_deagle" ) || StrEqual( sWeapon, "weapon_knife" ) )
		return Plugin_Continue;

	return Plugin_Handled;
}

public Action:OnWeaponCanSwitchTo(client, weapon)
{
	if ( !IsValidEntity(weapon) || !IsValidEdict(weapon) || !IsValidAlive(client) ||
	!p_DuelAccepted[client] )
		return Plugin_Continue;

	decl String:sWeapon[32];
	GetEdictClassname(weapon, sWeapon, sizeof(sWeapon));
	
	if ( !p_IsDueling[client] && StrEqual( sWeapon, "weapon_deagle") )
		return Plugin_Handled;

	return Plugin_Continue;
}

public Action:Say_Command(client, const String:command[], argc)
{
	if ( !IsValidAlive(client) )
		return Plugin_Continue;
	
	decl String:Text[192];
	decl String:szArg1[16]
	GetCmdArgString(Text, sizeof(Text));

	StripQuotes(Text)

	BreakString(Text, szArg1, sizeof(szArg1))
	
	if( StrEqual(szArg1, "!duel") || StrEqual(szArg1, "!wduel") )
	{
		if ( p_DuelAccepted[client] && !is_RoundEnded )
		{
			PrintToChat( client, "You are already dueling!");
		}
		else
		{
			OpenDuelMenu(client);
		}
	
		return Plugin_Handled;
	}
	
	return Plugin_Continue;
}
public OpenDuelMenu(client)
{
	new Handle:menu = CreateMenu(DuelMenu);

	SetMenuTitle(menu, "WDueling Menu");

	for( new i = 1 ; i <= MaxClients ; i++ )
	{
		if ( IsValidAlive( i ) && i != client )
		{
			decl String:DisplayItem[32], String: MenuItem[4];
			Format(DisplayItem, sizeof(DisplayItem), "%N", i);
			Format(MenuItem, sizeof(MenuItem), "%d", i);

			AddMenuItem( menu, MenuItem , DisplayItem );
		}
	}

	SetMenuExitButton(menu, true);
	
	DisplayMenu(menu, client, 20 );
}
public DuelMenu(Handle:menu, MenuAction:action, client, param2)
{
	if( action == MenuAction_Select )
	{	
		if ( is_RoundEnded ) return;
	
		decl String:info[32];
		
		GetMenuItem(menu, param2, info, sizeof(info));

		new target = StringToInt(info);

		if( IsValidAlive( target ) )
		{
			if ( p_DuelAccepted[target] )
			{
				PrintToChat( client, "%N уже участвует в дуэли!", target );
			}
			else
			{
				PrintToChat( client, "Предложение уже отправлено игроку %N!", target );
				PrintToChat( target, "%N предлагает вам дуэль!", client );

				p_DuelTarget[client] = target;
				p_DuelTarget[target] = client;

				DuelAcceptMenu(target);
			}
		}
		else
		{
			PrintToChat( client, "Ваш оппонент мертв!", client );

			OpenDuelMenu(client);
		}
	} 
	else if (action == MenuAction_End)	
	{
		CloseHandle(menu);
	}
}
public DuelAcceptMenu(target)
{
	new Handle:menu = CreateMenu(DuelAccept);

	SetMenuTitle(menu, "Вам предложили дуэль");

	AddMenuItem(menu, "duel_id", "Принять!")
	AddMenuItem(menu, "duel_id", "Зассал=)")

	SetMenuExitButton(menu, false);
	
	DisplayMenu(menu, target, 20 );
}
public DuelAccept(Handle:menu, MenuAction:action, target, param2)
{
	if( action == MenuAction_Select )
	{	
		if ( is_RoundEnded ) return;
	
		if( !IsValidAlive( target ) || !IsValidAlive( p_DuelTarget[target] ) )
			return;

		switch (param2)
		{
			case 0:
			{
				p_DuelAccepted[target] = true;
				p_DuelAccepted[p_DuelTarget[target]] = true;

				PrintToChat( target, "Вы согласились на дуэль!", p_DuelTarget[target] );
				PrintToChat( p_DuelTarget[target], "%N принял дуэль!", target );

				new Handle:data = CreateDataPack();
									
				WritePackCell(data, p_DuelTarget[target]);
				WritePackCell(data, target);

				DuelPrepareTimer[target] = CreateTimer( 2.0, Timer_PrepareDuel, data, TIMER_REPEAT|TIMER_FLAG_NO_MAPCHANGE);
				DuelFailTimer[target] = CreateTimer( GetConVarFloat(CvarDuelCheckTimer), Timer_FailDuel, data);
			}
			case 1:
			{
				PrintToChat( target, "Дуэль отменена с игроком %N!", p_DuelTarget[target] );
				PrintToChat( p_DuelTarget[target], "%N отказался от дуэли!", target );
			}
		}
	} 
	else if (action == MenuAction_End)	
	{
		CloseHandle(menu);
	}
}

public Action:Timer_PrepareDuel(Handle:timer, Handle:data)
{
	ResetPack(data);
	new client = ReadPackCell(data);
	new target = ReadPackCell(data);

	if( !IsValidAlive( target ) || !IsValidAlive( client ) )
		return Plugin_Stop;
		
	/*new Float:g_Vel_client[3], Float:g_Vel_target[3], Float:client_speed, Float:target_speed;
	GetEntPropVector(client, Prop_Data, "m_vecVelocity", g_Vel_client);
	GetEntPropVector(target, Prop_Data, "m_vecVelocity", g_Vel_target);
	client_speed = GetVectorLength( g_Vel_client );
	target_speed = GetVectorLength( g_Vel_target );*/

	if ( IsClientInView(target, client) && (GetEntityFlags(client) & FL_ONGROUND)
	&& (GetEntityFlags(target) & FL_ONGROUND) )
	{
		if ( DuelFailTimer[ target ] != INVALID_HANDLE )
		{
			CloseHandle( DuelFailTimer[ target ] );
			DuelFailTimer[ target ] = INVALID_HANDLE;
		}

		SetEntityMoveType(client, MOVETYPE_NONE);
		SetEntityMoveType(target, MOVETYPE_NONE);

		Strip_User_Weapons(client);
		Strip_User_Weapons(target);

		GivePlayerItem(client, "weapon_deagle");
		GivePlayerItem(target, "weapon_deagle");

		PrintToChat( target, "Дуэль началась,приготовтесь! Осталось %.1f", GetConVarFloat(CvarDuelStartTime) );
		PrintToChat( client, "Дуэль началась,приготовтесь! Осталось %.1f", GetConVarFloat(CvarDuelStartTime) );
		
		EmitSoundToClient( client, DUELING_SOUNDS[ GetRandomInt( 0, sizeof( DUELING_SOUNDS ) - 1 ) ], SOUND_FROM_PLAYER, SNDCHAN_AUTO, SNDLEVEL_NORMAL, SND_NOFLAGS, 0.8 );
		EmitSoundToClient( target, DUELING_SOUNDS[ GetRandomInt( 0, sizeof( DUELING_SOUNDS ) - 1 ) ], SOUND_FROM_PLAYER, SNDCHAN_AUTO, SNDLEVEL_NORMAL, SND_NOFLAGS, 0.8 );

		new Handle:data2 = CreateDataPack();
									
		WritePackCell(data2, client);
		WritePackCell(data2, target);

		DuelStartTimer[target] = CreateTimer( GetConVarFloat(CvarDuelStartTime), Timer_StartDuel, data2);

		return Plugin_Stop;
	}

	PrintToChat( target, "Ваш оппонент сейчас находится в движении или далеко от вас" );
	PrintToChat( client, "Ваш оппонент сейчас находится в движении или далеко от вас" );

	return Plugin_Continue;
}
public Action:Timer_FailDuel(Handle:timer, Handle:data)
{
	ResetPack(data);
	new client = ReadPackCell(data);
	new target = ReadPackCell(data);

	DuelFailTimer[ target ] = INVALID_HANDLE;

	if( !IsValidAlive( target ) || !IsValidAlive( client ) )
		return Plugin_Handled;

	if ( DuelPrepareTimer[ target ] != INVALID_HANDLE )
	{
		CloseHandle( DuelPrepareTimer[ target ] );
		DuelPrepareTimer[ target ] = INVALID_HANDLE;
	}

	p_DuelTarget[target] = 0;
	p_DuelTarget[client] = 0;

	p_DuelAccepted[target] = false;
	p_DuelAccepted[client] = false;

	PrintToChat( target, "Бедааа,вы не нашли вашу цель=(" );
	PrintToChat( client, "Бедааа,вы не нашли вашу цель=(" );

	return Plugin_Handled;
}
public Action:Timer_StartDuel(Handle:timer, Handle:data)
{
	ResetPack(data);
	new client = ReadPackCell(data);
	new target = ReadPackCell(data);

	DuelStartTimer[ target ] = INVALID_HANDLE;

	if( !IsValidAlive( target ) || !IsValidAlive( client ) )
		return Plugin_Handled;

	p_IsDueling[target] = true;
	p_IsDueling[client] = true;

	PrintToChat( target, "Достань пистолет и стреляй!" );
	PrintToChat( client, "Достань пистолет и стреляй!" );
	
	new Handle:data2 = CreateDataPack();
									
	WritePackCell(data2, client);
	WritePackCell(data2, target);
	
	DuelEndTimer[target] = CreateTimer( GetConVarFloat(CvarDuelEndTime) , Timer_EndDuel, data2);
	
	return Plugin_Handled;
}
public Action:Timer_EndDuel(Handle:timer, Handle:data)
{
	ResetPack(data);
	new client = ReadPackCell(data);
	new target = ReadPackCell(data);

	DuelEndTimer[ target ] = INVALID_HANDLE;

	if ( IsValidAlive( client ) )
	{
		p_DuelAccepted[client] = false;
		p_IsDueling[client] = false;
		p_DuelTarget[client] = 0;
				
		SetEntityMoveType(client, MOVETYPE_WALK);

		Strip_Wep_Deagle(client);

		if ( !StrEqual(WepBeforeDuel[client][CS_SLOT_PRIMARY], "NULL") )
			GivePlayerItem(client, WepBeforeDuel[client][CS_SLOT_PRIMARY]);
		if ( !StrEqual(WepBeforeDuel[client][CS_SLOT_SECONDARY], "NULL") )
			GivePlayerItem(client, WepBeforeDuel[client][CS_SLOT_SECONDARY]);
		
		PrintToChat( client, "Вы проиграли!" );
	}
	if ( IsValidAlive( target ) )
	{
		p_DuelAccepted[target] = false;
		p_IsDueling[target] = false;
		p_DuelTarget[target] = 0;
				
		SetEntityMoveType(target, MOVETYPE_WALK);

		Strip_Wep_Deagle(target);

		if ( !StrEqual(WepBeforeDuel[target][CS_SLOT_PRIMARY], "NULL") )
			GivePlayerItem(target, WepBeforeDuel[target][CS_SLOT_PRIMARY]);
		if ( !StrEqual(WepBeforeDuel[target][CS_SLOT_SECONDARY], "NULL") )
			GivePlayerItem(target, WepBeforeDuel[target][CS_SLOT_SECONDARY]);
	
		if ( DuelStartTimer[ target ] != INVALID_HANDLE )
		{
			CloseHandle( DuelStartTimer[ target ] );
			DuelStartTimer[ target ] = INVALID_HANDLE;
		}
		if ( DuelFailTimer[ target ] != INVALID_HANDLE )
		{
			CloseHandle( DuelFailTimer[ target ] );
			DuelFailTimer[ target ] = INVALID_HANDLE;
		}
		
		PrintToChat( target, "Вы проиграли!" );
	}
		
	return Plugin_Handled;
}

public Action:Strip_User_Weapons(client)
{
	if ( !IsValidAlive(client) )
		return Plugin_Continue;

	new wepIdx;
	for (new x = CS_SLOT_PRIMARY; x < CS_SLOT_C4 + 1; x++)
	{
		if ( x == CS_SLOT_PRIMARY && (wepIdx = GetPlayerWeaponSlot(client, x)) != -1 )
		{
			GetEdictClassname(wepIdx, WepBeforeDuel[client][CS_SLOT_PRIMARY], sizeof(WepBeforeDuel[][]));
		}
		else
			Format( WepBeforeDuel[client][CS_SLOT_PRIMARY], sizeof(WepBeforeDuel[][]), "NULL");

		if ( x == CS_SLOT_SECONDARY && (wepIdx = GetPlayerWeaponSlot(client, x)) != -1 )
		{
			GetEdictClassname(wepIdx, WepBeforeDuel[client][CS_SLOT_SECONDARY], sizeof(WepBeforeDuel[][]));
		}
		else
			Format( WepBeforeDuel[client][CS_SLOT_SECONDARY], sizeof(WepBeforeDuel[][]), "NULL");

		if ( x != 2 && (wepIdx = GetPlayerWeaponSlot(client, x)) != -1 )
		{  
			RemovePlayerItem(client, wepIdx);
			AcceptEntityInput(wepIdx, "Kill");
		}
	}

	return Plugin_Continue
}
public bool:IsClientInView(client, target)
{
	new Float:Origin[3];

	new bool:HitPlayer = false;
				
	new Float:targetorigin[3], Float:distancebetween;

	if ( IsValidAlive(target) )
	{
		GetClientAbsOrigin( client, Origin);
		GetClientAbsOrigin ( target, targetorigin );
		distancebetween = GetVectorDistance ( targetorigin, Origin );
							
		if ( distancebetween <= 400.0 )
		{
			HitPlayer = true
			return HitPlayer;
		}
							
		targetorigin[0] -= 20.0;
		targetorigin[2] -= 0.0;
							
		Origin[0] -= 20.0
		Origin[2] -= 0.0
					
		for(new pos = 0; pos <= 11; pos++) // Check for position
		{
			targetorigin[0] += 2.5;
			targetorigin[2] += 6.0;
								
			Origin[0] += 2.5;
			Origin[2] += 6.0;
							
			//static BeamColor[4] = {155, 55, 55 , 90}
		
			//TE_SetupBeamPoints( Origin, targetorigin, LaserCache, 0, 0, 0, 1.1, 16.0, 8.0, 0, 0.5, BeamColor, 0);
			//TE_SendToAll();
							
			new Handle:trace = TR_TraceRayFilterEx( Origin, targetorigin, MASK_SOLID, RayType_EndPoint, TraceEntityFilterPlayer2);
							
			if ( !TR_DidHit(trace) )
			{
				HitPlayer = true
			}

			CloseHandle(trace);
		}
	}

	return HitPlayer;
}

public bool:TraceEntityFilterPlayer2(entity, iContentsMask)
{
	return entity > MaxClients;
}
public IsValidClient( client )
{
	if ( !( 1 <= client <= MaxClients ) || !IsClientInGame(client) )
		return false;
	
	return true;
}
public IsValidAlive( client )
{
	if ( !IsValidClient(client) || !IsPlayerAlive(client) )
		return false;
	
	return true;
}
public Action:Strip_Wep_Deagle(client)
{
	new wepIdx;
	for (new x = CS_SLOT_PRIMARY; x < CS_SLOT_SECONDARY + 1; x++)
	{
		if ( x == CS_SLOT_SECONDARY && (wepIdx = GetPlayerWeaponSlot(client, x)) != -1 )
		{  
			RemovePlayerItem(client, wepIdx);
			AcceptEntityInput(wepIdx, "Kill");
		}
	}
}
// Set active wep
// new weapon = GetPlayerWeaponSlot(client, 2);
// SetEntPropEnt(client, Prop_Send, "m_hActiveWeapon", weapon); 